@@ -1,14 +1,14 @@
-# Copyrights 2007-2014 by [Mark Overmeer].
+# Copyrights 2007-2016 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 2.01.
+# Pod stripped from pm file by OODoc 2.02.
 
 use warnings;
 use strict;
 
 package Log::Report;
 use vars '$VERSION';
-$VERSION = '1.05';
+$VERSION = '1.13';
 
 use base 'Exporter';
 
@@ -16,6 +16,7 @@ use List::Util         qw/first/;
 use Scalar::Util       qw/blessed/;
 
 use Log::Report::Util;
+my $lrm = 'Log::Report::Message';
 
 ### if you change anything here, you also have to change Log::Report::Minimal
 my @make_msg         = qw/__ __x __n __nx __xn N__ N__n N__w/;
@@ -54,7 +55,7 @@ require Log::Report::Dispatcher::Try;
 #, translator => Log::Report::Translator::POT->new(charset => 'utf-8');
 textdomain 'log-report';
 
-dispatcher PERL => 'default', accept => 'NOTICE-';
+my $default_dispatcher = dispatcher PERL => 'default', accept => 'NOTICE-';
 
 
 sub report($@)
@@ -62,16 +63,23 @@ sub report($@)
     my $reason = shift;
     my $stop = exists $opts->{is_fatal} ? $opts->{is_fatal} : is_fatal $reason;
 
-    my $try  = $nested_tries[-1];
-    my @disp = defined $try && $stop ? () : @{$reporter->{needs}{$reason}||[]};
-    push @disp, $try if defined $try && $try->needs($reason);
-
-    # return when no-one needs it: skip unused trace() fast!
-    @disp || $stop or return;
+    my @disp;
+    if(defined(my $try = $nested_tries[-1]))
+    {   push @disp, @{$reporter->{needs}{$reason}||[]}
+            unless $stop || $try->hides($reason);
+        push @disp, $try if $try->needs($reason);
+    }
+    else
+    {   @disp = @{$reporter->{needs}{$reason} || []};
+    }
 
     is_reason $reason
         or error __x"token '{token}' not recognized as reason", token=>$reason;
 
+    # return when no-one needs it: skip unused trace() fast!
+    @disp || $stop
+        or return;
+
     $opts->{errno} ||= $!+0 || $? || 1
         if use_errno($reason) && !defined $opts->{errno};
 
@@ -97,17 +105,17 @@ sub report($@)
     $opts->{location} ||= Log::Report::Dispatcher->collectLocation;
 
     my $exception;
-    if(UNIVERSAL::isa($message, 'Log::Report::Message'))
-    {   @_==0 or error __x"a message object is reported with more parameters";
+    if(!blessed $message)
+    {   # untranslated message into object
+        @_%2 and error __x"odd length parameter list with '{msg}'", msg => $message;
+        $message  = $lrm->new(_prepend => $message, @_);
     }
-    elsif(UNIVERSAL::isa($message, 'Log::Report::Exception'))
+    elsif($message->isa('Log::Report::Exception'))
     {   $exception = $message;
         $message   = $exception->message;
     }
-    else
-    {   # untranslated message into object
-        @_%2 and error __x"odd length parameter list with '{msg}'", msg => $message;
-        $message = Log::Report::Message->new(_prepend => $message, @_);
+    elsif($message->isa('Log::Report::Message'))
+    {   @_==0 or error __x"a message object is reported with more parameters";
     }
 
     if(my $to = $message->to)
@@ -147,24 +155,45 @@ sub report($@)
 }
 
 
+my %disp_actions = map +($_ => 1), qw/
+  close find list disable enable mode needs filter active-try do-not-reopen
+  /;
+
+my $reopen_disp = 1;
+
 sub dispatcher($@)
-{   if($_[0] !~ m/^(?:close|find|list|disable|enable|mode|needs|filter)$/)
+{   if(! $disp_actions{$_[0]})
     {   my ($type, $name) = (shift, shift);
-        my $disp = Log::Report::Dispatcher->new($type, $name
-          , mode => $default_mode, @_);
-        defined $disp or return;  # use defined, because $disp is overloaded
 
         # old dispatcher with same name will be closed in DESTROY
-        $reporter->{dispatchers}{$name} = $disp;
+        my $disps = $reporter->{dispatchers};
+ 
+        if(!$reopen_disp)
+        {   my $has = first {$_->name eq $name} @$disps;
+            if(defined $has && $has ne $default_dispatcher)
+            {   trace "not reopening $name";
+                return $has;
+            }
+        }
+
+        my @disps = grep $_->name ne $name, @$disps;
+        trace "reopening dispatcher $name" if @disps != @$disps;
+
+        my $disp = Log::Report::Dispatcher
+           ->new($type, $name, mode => $default_mode, @_);
+
+        push @disps, $disp if $disp;
+        $reporter->{dispatchers} = \@disps;
+
         _whats_needed;
-        return ($disp);
+        return $disp ? ($disp) : undef;
     }
 
     my $command = shift;
     if($command eq 'list')
     {   mistake __"the 'list' sub-command doesn't expect additional parameters"
            if @_;
-        my @disp = values %{$reporter->{dispatchers}};
+        my @disp = @{$reporter->{dispatchers}};
         push @disp, $nested_tries[-1] if @nested_tries;
         return @disp;
     }
@@ -179,40 +208,52 @@ sub dispatcher($@)
     {   my $code = shift;
         error __"the 'filter' sub-command needs a CODE reference"
             unless ref $code eq 'CODE';
-        my %names = map { ($_ => 1) } @_;
+        my %names = map +($_ => 1), @_;
         push @{$reporter->{filters}}, [ $code, \%names ];
         return ();
     }
+    if($command eq 'active-try')
+    {   return $nested_tries[-1];
+    }
+    if($command eq 'do-not-reopen')
+    {   $reopen_disp = 0;
+        return ();
+    }
 
     my $mode     = $command eq 'mode' ? shift : undef;
 
     my $all_disp = @_==1 && $_[0] eq 'ALL';
-    my @disps    = $all_disp ? keys %{$reporter->{dispatchers}} : @_;
-
-    my @dispatchers = grep defined, @{$reporter->{dispatchers}}{@disps};
-    @dispatchers or return;
+    my $disps    = $reporter->{dispatchers};
+    my @disps;
+    if($all_disp) { @disps = @$disps }
+    else
+    {   # take the dispatchers in the specified order.  Both lists
+        # are small, so O(x²) is small enough
+        for my $n (@_) { push @disps, grep $_->name eq $n, @$disps }
+    }
 
     error __"only one dispatcher name accepted in SCALAR context"
-        if @dispatchers > 1 && !wantarray && defined wantarray;
+        if @disps > 1 && !wantarray && defined wantarray;
 
     if($command eq 'close')
-    {   delete @{$reporter->{dispatchers}}{@disps};
-        $_->close for @dispatchers;
+    {   my %kill = map +($_->name => 1), @disps;
+        @$disps  = grep !$kill{$_->name}, @$disps;
+        $_->close for @disps;
     }
-    elsif($command eq 'enable')  { $_->_disabled(0) for @dispatchers }
-    elsif($command eq 'disable') { $_->_disabled(1) for @dispatchers }
+    elsif($command eq 'enable')  { $_->_disabled(0) for @disps }
+    elsif($command eq 'disable') { $_->_disabled(1) for @disps }
     elsif($command eq 'mode')
     {    Log::Report::Dispatcher->defaultMode($mode) if $all_disp;
-         $_->_set_mode($mode) for @dispatchers;
+         $_->_set_mode($mode) for @disps;
     }
 
     # find does require reinventarization
-    _whats_needed unless $command eq 'find';
+    _whats_needed if $command ne 'find';
 
-    wantarray ? @dispatchers : $dispatchers[0];
+    wantarray ? @disps : $disps[0];
 }
 
-END { $_->close for grep defined, values %{$reporter->{dispatchers}} }
+END { $_->close for @{$reporter->{dispatchers}} }
 
 # _whats_needed
 # Investigate from all dispatchers which reasons will need to be
@@ -221,7 +262,7 @@ END { $_->close for grep defined, values %{$reporter->{dispatchers}} }
 
 sub _whats_needed()
 {   my %needs;
-    foreach my $disp (values %{$reporter->{dispatchers}})
+    foreach my $disp (@{$reporter->{dispatchers}})
     {   push @{$needs{$_}}, $disp for $disp->needs;
     }
     $reporter->{needs} = \%needs;
@@ -238,6 +279,9 @@ sub try(&@)
     my $disp = Log::Report::Dispatcher::Try->new(TRY => 'try', @_);
     push @nested_tries, $disp;
 
+    # user's __DIE__ handlers would frustrate the exception mechanism
+    local $SIG{__DIE__};
+
     my ($ret, @ret);
     if(!defined wantarray)  { eval { $code->() } } # VOID   context
     elsif(wantarray) { @ret = eval { $code->() } } # LIST   context
@@ -277,7 +321,7 @@ sub panic(@)   {report PANIC   => @_}
 sub _default_domain(@) { pkg2domain $_[0] }
 
 sub __($)
-{   Log::Report::Message->new
+{   $lrm->new
       ( _msgid  => shift
       , _domain => _default_domain(caller)
       );
@@ -290,7 +334,7 @@ sub __x($@)
         where => join(' line ', (caller)[1,2]);
 
     my $msgid = shift;
-    Log::Report::Message->new
+    $lrm->new
      ( _msgid  => $msgid
      , _expand => 1
      , _domain => _default_domain(caller)
@@ -301,7 +345,7 @@ sub __x($@)
 
 sub __n($$$@)
 {   my ($single, $plural, $count) = (shift, shift, shift);
-    Log::Report::Message->new
+    $lrm->new
      ( _msgid  => $single
      , _plural => $plural
      , _count  => $count
@@ -313,7 +357,7 @@ sub __n($$$@)
 
 sub __nx($$$@)
 {   my ($single, $plural, $count) = (shift, shift, shift);
-    Log::Report::Message->new
+    $lrm->new
      ( _msgid  => $single
      , _plural => $plural
      , _count  => $count
@@ -326,7 +370,7 @@ sub __nx($$$@)
 
 sub __xn($$$@)   # repeated for prototype
 {   my ($single, $plural, $count) = (shift, shift, shift);
-    Log::Report::Message->new
+    $lrm->new
      ( _msgid  => $single
      , _plural => $plural
      , _count  => $count
@@ -396,6 +440,13 @@ sub import(@)
         }
     }
 
+    if(my $msg_class = delete $opts{message_class})
+    {   $msg_class->isa($lrm)
+            or error __x"message_class {class} does not extend {base}"
+                 , base => $lrm, class => $msg_class;
+        $lrm = $msg_class;
+    }
+
     $class->export_to_level(1+$to_level, undef, @export);
 
     ### Log::Report::Domain configuration
