@@ -1,13 +1,13 @@
-# Copyrights 2007-2014 by [Mark Overmeer].
+# Copyrights 2007-2016 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 2.01.
+# Pod stripped from pm file by OODoc 2.02.
 use warnings;
 use strict;
 
 package Log::Report::Die;
 use vars '$VERSION';
-$VERSION = '1.05';
+$VERSION = '1.13';
 
 use base 'Exporter';
 
@@ -21,6 +21,7 @@ sub die_decode($)
     @text or return ();
     chomp $text[-1];
 
+    # Try to catch the error directly, to remove it from the error text
     my %opt    = (errno => $! + 0);
     my $err    = "$!";
 
@@ -37,13 +38,14 @@ sub die_decode($)
     $text[0] =~ s/\s*[.:;]?\s*$err\s*$//  # the $err is translation sensitive
         or delete $opt{errno};
 
-    my $msg = shift @text;
-    length $msg or $msg = 'stopped';
+    my @msg = shift @text;
+    length $msg[0] or $msg[0] = 'stopped';
 
     my @stack;
     foreach (@text)
-    {   push @stack, [ $1, $2, $3 ]
-            if m/^\s*(.*?)\s+called at (.*?) line (\d+)\s*$/;
+    {   if(m/^\s*(.*?)\s+called at (.*?) line (\d+)\s*$/)
+             { push @stack, [ $1, $2, $3 ] }
+        else { push @msg, $_ }
     }
     $opt{stack}   = \@stack;
     $opt{classes} = [ 'perl', (@stack ? 'confess' : 'die') ];
@@ -52,7 +54,7 @@ sub die_decode($)
       = @{$opt{stack}} ? ($opt{errno} ? 'ALERT' : 'PANIC')
       :                  ($opt{errno} ? 'FAULT' : 'ERROR');
 
-    ($dietxt, \%opt, $reason, $msg);
+    ($dietxt, \%opt, $reason, join("\n",@msg));
 }
 
 "to die or not to die, that's the question";
