@@ -1,13 +1,13 @@
-# Copyrights 2007-2014 by [Mark Overmeer].
+# Copyrights 2007-2016 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 2.01.
+# Pod stripped from pm file by OODoc 2.02.
 use warnings;
 use strict;
 
 package Log::Report::Dispatcher;
 use vars '$VERSION';
-$VERSION = '1.05';
+$VERSION = '1.13';
 
 
 use Log::Report 'log-report';
@@ -30,8 +30,10 @@ my %modes = (NORMAL => 0, VERBOSE => 1, ASSERT => 2, DEBUG => 3
 my @default_accept = ('NOTICE-', 'INFO-', 'ASSERT-', 'ALL');
 my %always_loc = map +($_ => 1), qw/ASSERT ALERT FAILURE PANIC/;
 
-my %predef_dispatchers = map { (uc($_) => __PACKAGE__.'::'.$_) }
-   qw/File Perl Syslog Try Callback Log4perl/;
+my %predef_dispatchers = map +(uc($_) => __PACKAGE__.'::'.$_)
+  , qw/File Perl Syslog Try Callback Log4perl/;
+
+my @skip_stack = sub { $_[0][0] =~ m/^Log\:\:Report(?:\:\:|$)/ };
 
 
 sub new(@)
@@ -146,6 +148,7 @@ sub needs(;$)
     first {$need eq $_} @$needs;
 }
 
+#-----------
 
 sub log($$$$)
 {   panic "method log() must be extended per back-end";
@@ -191,7 +194,8 @@ sub translate($$$)
       = ( defined $format
         ? __x($format, message => $msg->toString , reason => $r, error => $e)
         : $msg
-        )->toString . "\n";
+        )->toString;
+    $text =~ s/\n*\z/\n/;
 
     if($show_loc)
     {   if(my $loc = $opts->{location} || $self->collectLocation)
@@ -223,14 +227,7 @@ sub translate($$$)
 
 sub collectStack($)
 {   my ($thing, $max) = @_;
-
-    my ($nest, $sub) = (1, undef);
-    do { $sub = (caller $nest++)[3] }
-    while(defined $sub && $sub ne 'Log::Report::report');
-    defined $sub or $nest = 1;  # not found
-
-    # skip syntax==SHORT routine entries
-#   $nest++ if defined $sub && $sub =~ m/^Log\:\:Report\:\:/;
+    my $nest = $thing->skipStack;
 
     # special trick by Perl for Carp::Heavy: adds @DB::args
   { package DB;    # non-blank before package to avoid problem with OODoc
@@ -249,22 +246,29 @@ sub collectStack($)
 }
 
 
-sub collectLocation()
+sub addSkipStack(@)
+{   my $thing = shift;
+    push @skip_stack, @_;
+    $thing;
+}
+
+
+sub skipStack()
 {   my $thing = shift;
     my $nest  = 1;
-    my @args;
+    my $args;
 
-    do {@args = caller $nest++}
-    until $args[3] eq 'Log::Report::report';  # common entry point
+    do { $args = [caller ++$nest] }
+    while @$args && first {$_->($args)} @skip_stack;
 
-    # skip syntax==SHORT routine entries
-    @args = caller $nest++
-        if +(caller $nest)[3] =~ m/^Log\:\:Report\:\:[^:]*$/;
-
-    @args ? \@args : undef;
+    # do not count my own stack level in!
+    @$args ? $nest-1 : 1;
 }
 
 
+sub collectLocation() { [caller shift->skipStack] }
+
+
 sub stackTraceLine(@)
 {   my ($thing, %args) = @_;
 
@@ -348,5 +352,6 @@ sub stackTraceParam($$$)
     qq{"$escaped"};
 }
 
+#------------
 
 1;
