@@ -1,13 +1,13 @@
-# Copyrights 2007-2013 by [Mark Overmeer].
+# Copyrights 2007-2016 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 2.01.
+# Pod stripped from pm file by OODoc 2.02.
 use warnings;
 use strict;
 
 package Log::Report::Dispatcher::Callback;
 use vars '$VERSION';
-$VERSION = '0.998';
+$VERSION = '1.13';
 
 use base 'Log::Report::Dispatcher';
 
@@ -28,7 +28,7 @@ sub init($)
 sub callback() {shift->{callback}}
 
 
-sub log($$$)
+sub log($$$$)
 {   my $self = shift;
     $self->{callback}->($self, @_);
 }
