@@ -1,13 +1,13 @@
-# Copyrights 2007-2014 by [Mark Overmeer].
+# Copyrights 2007-2016 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 2.01.
+# Pod stripped from pm file by OODoc 2.02.
 use warnings;
 use strict;
 
 package Log::Report::Domain;
 use vars '$VERSION';
-$VERSION = '1.05';
+$VERSION = '1.13';
 
 use base 'Log::Report::Minimal::Domain';
 
@@ -45,7 +45,7 @@ sub configure(%)
     $self->SUPER::configure(%args);
 
     my $transl = $args{translator} || Log::Report::Translator->new;
-    $transl    =  Log::Report::Translator->new(@$transl)
+    $transl    =  Log::Report::Translator->new(%$transl)
         if ref $transl eq 'HASH';
 
     !blessed $transl || $transl->isa('Log::Report::Translator')
@@ -79,11 +79,23 @@ sub configure(%)
 
 
 sub setContext(@)
+{   my $self = shift;
+    my $cr   = $self->contextRules  # ignore context if no rules given
+        or error __x"you need to configure context_rules before setContext";
+
+    $self->{LRD_ctxt_def} = $cr->needDecode(set => @_);
+}
+
+
+sub updateContext(@)
 {   my $self = shift;
     my $cr   = $self->contextRules  # ignore context if no rules given
         or return;
 
-    $self->{LRD_ctxt_def} = $cr->needDecode(setContext => shift);
+    my $rules = $cr->needDecode(update => @_);
+    my $r = $self->{LRD_ctxt_def} ||= {};
+    @{$r}{keys %$r} = values %$r;
+    $r;
 }
 
 
