@@ -1,20 +1,24 @@
-# Copyrights 2007-2013 by [Mark Overmeer].
+# Copyrights 2007-2016 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 2.01.
+# Pod stripped from pm file by OODoc 2.02.
 use warnings;
 use strict;
 
 package Log::Report::Exception;
 use vars '$VERSION';
-$VERSION = '0.998';
+$VERSION = '1.13';
 
 
-use Log::Report 'log-report';
-use POSIX  qw/locale_h/;
+use Log::Report      'log-report';
+use Log::Report::Util qw/is_fatal to_html/;
+use POSIX             qw/locale_h/;
+use Scalar::Util      qw/blessed/;
 
 
-use overload '""' => 'toString';
+use overload
+    '""'     => 'toString'
+  , fallback => 1;
 
 
 sub new($@)
@@ -33,15 +37,15 @@ sub reason(;$)
 }
 
 
-sub isFatal() { Log::Report->isFatal(shift->{reason}) }
+sub isFatal() { is_fatal shift->{reason} }
 
 
 sub message(;$)
 {   my $self = shift;
     @_ or return $self->{message};
     my $msg = shift;
-    UNIVERSAL::isa($msg, 'Log::Report::Message')
-        or panic __x"message() of exception expects Log::Report::Message";
+    blessed $msg && $msg->isa('Log::Report::Message')
+        or panic "message() of exception expects Log::Report::Message";
     $self->{message} = $msg;
 }
 
@@ -55,16 +59,15 @@ sub throw(@)
 
     my $reason;
     if($reason = delete $opts->{reason})
-    {   $opts->{is_fatal} = Log::Report->isFatal($reason)
+    {   $self->{reason} = $reason;
+        $opts->{is_fatal} = is_fatal $reason
             unless exists $opts->{is_fatal};
     }
     else
     {   $reason = $self->{reason};
     }
 
-    $opts->{stack} = Log::Report::Dispatcher->collectStack
-        if $opts->{stack} && @{$opts->{stack}};
-
+    $opts->{stack} ||= Log::Report::Dispatcher->collectStack;
     report $opts, $reason, $self;
 }
 
@@ -72,13 +75,16 @@ sub throw(@)
 sub PROPAGATE($$) {shift}
 
 
-sub toString()
-{   my $self = shift;
+sub toString(;$)
+{   my ($self, $locale) = @_;
     my $msg  = $self->message;
-    lc($self->{reason}) . ': ' . (ref $msg ? $msg->toString : $msg) . "\n";
+    lc($self->{reason}).': '.(ref $msg ? $msg->toString($locale) : $msg)."\n";
 }
 
 
+sub toHTML(;$) { to_html($_[0]->toString($_[1])) }
+
+
 sub print(;$)
 {   my $self = shift;
     (shift || *STDERR)->print($self->toString);
