@@ -1,18 +1,19 @@
-# Copyrights 2007-2014 by [Mark Overmeer].
+# Copyrights 2007-2016 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 2.01.
+# Pod stripped from pm file by OODoc 2.02.
 use warnings;
 use strict;
 
 package Log::Report::Message;
 use vars '$VERSION';
-$VERSION = '1.05';
+$VERSION = '1.13';
 
 
 use Log::Report 'log-report';
-use POSIX      qw/locale_h/;
-use List::Util qw/first/;
+use POSIX             qw/locale_h/;
+use List::Util        qw/first/;
+use Log::Report::Util qw/to_html/;
 
 # Work-around for missing LC_MESSAGES on old Perls and Windows
 { no warnings;
@@ -80,6 +81,7 @@ sub fromTemplateToolkit($$;@)
       , %$args, _expand => 1, _domain => $domain);
 }
 
+#----------------
 
 sub prepend() {shift->{_prepend}}
 sub msgid()   {shift->{_msgid}}
@@ -103,6 +105,7 @@ sub to(;$)
 
 sub valueOf($) { $_[0]->{$_[1]} }
 
+#--------------
 
 sub inClass($)
 {   my @classes = shift->classes;
@@ -131,10 +134,8 @@ sub toString(;$)
   
     defined $text or return ();
 
-    if($self->{_expand})
-    {    my $re   = join '|', map quotemeta, keys %$self;
-         $text    =~ s/\{($re)(\%[^}]*)?\}/$self->_expand($1,$2)/ge;
-    }
+    $text  =~ s/\{([^%}]+)(\%[^}]*)?\}/$self->_expand($1,$2)/ge
+        if $self->{_expand};
 
     $text  = "$self->{_prepend}$text"
         if defined $self->{_prepend};
@@ -150,7 +151,7 @@ sub toString(;$)
 
 sub _expand($$)
 {   my ($self, $key, $format) = @_;
-    my $value = $self->{$key};
+    my $value = $self->{$key} // $self->{_context}{$key};
 
     $value = $value->($self)
         while ref $value eq 'CODE';
@@ -173,6 +174,11 @@ sub _expand($$)
 }
 
 
+my %tohtml = qw/  > gt   < lt   " quot  & amp /;
+
+sub toHTML(;$) { to_html($_[0]->toString($_[1])) }
+
+
 sub untranslated()
 {  my $self = shift;
      (defined $self->{_prepend} ? $self->{_prepend} : '')
@@ -192,5 +198,6 @@ sub concat($;$)
     ref($self)->new(%$self, _append => $what);
 }
 
+#----------------
 
 1;
